/* ------------------------------------------------------------------------
   file:   cmdline.c - Command line processor
   description: This code processes the command line using getopt.
   ------------------------------------------------------------------------ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#define _GNU_SOURCE
#include <getopt.h>

#include "cmdline.h"
#include "version.h"

static struct option long_options[] = 
	{
		{"help",0,0,'h'},
		{"file",1,0,'f'},
		{"volthreshold",1,0,'r'},
		{"intervaltime",1,0,'t'},
		{"outdir",1,0,'d'},
		{"verbose",0,0,'v'},
		{"mintracklen",1,0,'m'},
		{0,0,0,0}
	};

void cmdline(int argc, char *argv[], char** infile,  char** outdir, int *volthreshold, int *intervaltime, int* mintracklen, int* verbose) {
	int c=0;
	int option_index = 0;

	*infile = NULL;
	*outdir = NULL;
	
	*volthreshold = 750;
	*intervaltime = 1;
	*verbose = 0;
	*mintracklen = 60;
	
	do {
		c = getopt_long(argc, argv, "h t: m: f: d: r: v", long_options, &option_index);
		
		switch(c) {
		case 'h':
			fprintf(stderr,"%s-%s ", EXE_NAME, VERSION);
			/*fprintf(stderr,"\nBug reports to toby@earth.li");*/
			fprintf(stderr,"\nUsage: %s -f <input.wav>", EXE_NAME);
			fprintf(stderr,"\noptions:");	
			fprintf(stderr,"\n-h\t\t--help\t\tthis help");
			fprintf(stderr,"\n-f file\t\t--file\t\twav file to split");
			fprintf(stderr,"\n-r <0-32767>\t--volthreshold\tmaximum volume of fuzz (default 750)");
			fprintf(stderr,"\n-t <1-999999>\t--intervaltime\tminimum length of inter-track pauses, in seconds\n\t\t\t\t(default 1)");
			fprintf(stderr,"\n-m <0-999999>\t--mintracklen\tminimum length of an output track, in seconds\n\t\t\t\t(default 60)");
			fprintf(stderr,"\n-d directory\t--outdir\ttarget output directory (default \".\")");
			fprintf(stderr,"\n-v\t\t--verbose\tenable increased verbosity (default off)");
			fprintf(stderr,"\n\n");
			exit(-1);
			break;
			
		case 'r':
			if (sscanf(optarg, "%d", volthreshold) < 1 || *volthreshold <0 || *volthreshold>32767) {
				fprintf(stderr, "Invalid volume threshold (%s)\n", optarg);
				exit(-1);
			}
			break;
			
		case 't':
			if (sscanf(optarg, "%d", intervaltime) < 1 || *intervaltime < 1 || *intervaltime > 999999)	{
				fprintf(stderr, "Invalid inter-track interval (%s)\n",optarg);
				exit(-1);
			}
			break;

		case 'm':
			if (sscanf(optarg,"%d", mintracklen) < 1 || *mintracklen < 0 || *mintracklen > 999999) {
				fprintf(stderr,"Invalid minimum track length (%s)\n",optarg);
				exit(-1);
			}
			break;

		case 'f':
			*infile = strdup(optarg);
			break;
		
		case 'd':
		 	{
				struct stat dirstat;
				memset(&dirstat, 0, sizeof(dirstat));
				*outdir = strdup(optarg);
				
				if (stat(*outdir, &dirstat) || !(dirstat.st_mode & S_IFDIR)) {
					fprintf(stderr, "Output directory \"%s\" is not a valid directory.\n", *outdir);
					exit(-1);
				}
		    }
			break;

		case 'v':
			*verbose = 1;
			break;
			
		case '?': /* unknown argument */
		case ':': /* parameter missing */
			exit(-1);
			break;
		case -1: /* end of options*/			
			break;
		default:
			fprintf(stderr, "Unexpected logic path encountered. (%d)\n", c);
			exit(-1);
			break;
		}
	} while (c != -1);

	if (*infile == NULL) {
		fprintf(stderr, "No file specified, try --help\n");
		exit(1);
	}

	if (*outdir == NULL) {
		*outdir = strdup(".");
	}
}

